globals [
  percent-unhappy    ; perc of unhappy turtles
  percent-unhappy-1    ; perc of unhappy turtles of group 1
  percent-unhappy-2    ; perc of unhappy turtles of group 2
  segregation-1        ; measure of segr. for group 1
  segregation-2        ; measure of segr. for group 2
  color1                ; color of group 1
  color2                ; color of group 2
]

turtles-own [
  happy            ; true if turtle has at least percent-similar-wanted of similar in its neighbor
  similar-nearby   ; counts neighbors of the same color
  total-nearby     ; counts the n. of neighbors
]

to setup
  ca                ; clear all (not really useful explanation)
  set color1 45      ; yellow
  set color2 85      ; cyan
  let number-of-people int(world-width * world-height * (1 - perc-vacant-space / 100))
  ask n-of number-of-people patches [
    sprout 1 [set color color2]
  ]
  ask n-of int(number-of-people * perc-group1 / 100) turtles [
    set color color1
  ]
  update
end

to update
  show-happiness
  update-globals
  do-plot
end

to show-happiness
  ask turtles [
    compute-happiness
  ]
end

to compute-happiness
  set total-nearby count turtles-on neighbors
  set similar-nearby count (turtles-on neighbors) with [color = [color] of myself]
  ifelse (total-nearby = 0) [
    set happy true
  ] [
    set happy (similar-nearby / total-nearby >= perc-similar-wanted / 100)
  ]
  ifelse (happy = true) [
    set shape "face happy"
  ] [
    set shape "face sad"
  ]
end

to go
  if not any? turtles with [not happy] [stop]
  move-unhappy
  update
end

to move-unhappy
  ask turtles with [not happy] [ random-move ]
end

to random-move
  ; find an empty patches
  setxy random-pxcor random-pycor
  if any? other turtles-here [random-move]
end

to update-globals
  set percent-unhappy (count turtles with [not happy] / count turtles)   ; perc of unhappy turtles
  set percent-unhappy-1 (count turtles with [not happy and color = color1] / count turtles with [color = color1])  ; perc of unhappy turtles of color 1
  set percent-unhappy-2 (count turtles with [not happy and color = color2] / count turtles with [color = color2])   ; perc of unhappy turtles of color 2
  set segregation-1 mean [count (turtles-on neighbors) with [color = color1]] of turtles with [color = color1]
  set segregation-2 mean [count (turtles-on neighbors) with [color = color2]] of turtles with [color = color2]
end

to do-plot
  set-current-plot "Unhappy"
  set-current-plot-pen "Total"
  plot percent-unhappy
  set-current-plot-pen "1"
  plot percent-unhappy-1
  set-current-plot-pen "2"
  plot percent-unhappy-2
  set-current-plot "Segregation"
  set-current-plot-pen "seg1"
  plot segregation-1
  set-current-plot-pen "seg2"
  plot segregation-2
end
@#$#@#$#@
GRAPHICS-WINDOW
328
28
830
551
20
20
12.0
1
10
1
1
1
0
0
0
1
-20
20
-20
20
0
0
1
ticks

CC-WINDOW
5
569
1229
664
Command Center
0

SLIDER
14
15
161
48
perc-vacant-space
perc-vacant-space
5
90
10
5
1
NIL
HORIZONTAL

BUTTON
227
23
291
56
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

MONITOR
13
261
91
306
Turtles 1&2
count turtles
3
1
11

SLIDER
12
102
161
135
perc-group1
perc-group1
0
100
50
10
1
NIL
HORIZONTAL

MONITOR
13
148
71
193
 turtles 1
count turtles with [color = color1]
3
1
11

MONITOR
12
206
71
251
turtles 2
count turtles with [color = color2]
3
1
11

SLIDER
13
59
161
92
perc-similar-wanted
perc-similar-wanted
10
90
75
5
1
NIL
HORIZONTAL

BUTTON
227
73
290
106
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

PLOT
11
325
290
555
Unhappy
time
%
0.0
10.0
0.0
1.0
true
false
PENS
"Total" 1.0 0 -16777216 true
"1" 1.0 0 -1184463 true
"2" 1.0 0 -11221820 true

MONITOR
78
148
176
193
% unhappy (1)
percent-unhappy-1 * 100
3
1
11

MONITOR
78
205
175
250
% unhappy (2)
percent-unhappy-2 * 100
3
1
11

MONITOR
99
261
173
306
% unhappy
percent-unhappy
3
1
11

PLOT
870
37
1220
299
Segregation
NIL
NIL
0.0
10.0
0.0
10.0
true
false
PENS
"seg1" 1.0 0 -955883 true
"seg2" 1.0 0 -13345367 true

MONITOR
881
330
1022
375
Segregation of group 1
segregation-1
3
1
11

MONITOR
891
409
1032
454
Segregation of group 2
segregation-2
3
1
11

@#$#@#$#@
WHAT IS IT?
-----------
This model is a simple implementation of the Schelling model of segregation. For more information, you can refer to the original article

Thomas C. Schelling, Dynamic models of segregation, Journal of Mathematical Sociology, 1971, Vol. 1, pp 143-186

The model presented here is the two dimensional version of the model. Many other resources can be found Googling "schelling segregation model".

HOW IT WORKS
------------
The population lives on a square space. It is assumed that each agent belongs either to group 1 or 2. Agent of different groups are displayed in different colors.

At the beginning the population is created and displaced on the square, based on two parameters:
- perc-vacant-space is the percentage of patches that is left free, to allow for movements of people. As each space is either free or occupied by a single agent, this parameter completely determines the number of agents in the simulation;
- perc-color1 is the percentage of population of group1 on the total number of agents. As in this model there are only two populations, this parameter determines also the number of agents of population of group 2.

Every agent can be happy or unhappy depending on the number of neighbors of the same group. At each step, each agent determines its state of happiness by counting the number of neighbors of the same group and dividing it by the total number of neighbors. If this fraction is greater or equal to the perc-similar-wanted the agent is happy, otherwise is unhappy. Its state is reflected in the shape of its face.

Each tick of the model consists of two main steps:
1) if all agents are happy, the model stops; otherwise go to step 2
2) unhappy agents are moved to a random free location in the space; go to step 1

HOW TO USE IT
-------------
The usage is quite simple. User modifies the three parameters (perc-vacant-space, perc-color1 and perc-similar-wanted) and the presses setup to create the world.

Then one can press go to see the evolution of the model. If the model reaches equilibrium (that is all agents are happy) it automatically stops. If the model continuously loops, the user can stop it clicking on the go button.

Monitors and graphs display information about the populations.

THINGS TO NOTICE
----------------
The model implemented is a slight variation of the original Schelling model. In the article, Schelling defined the rules of movement this way:

"... to move to the nearest satisfactory square, with 'nearest' measured by the number of squares one traverses horizontally and vertically. ..."

So, Schelling's prescription has agents choose their destination among happy ones, provided they exists, starting the search from closer patches. The implementation of this rule in Netlogo is possible but requires some complex coding and for this reason another algorithm was implemented. 

In this implementation unhappy agents move to a random patch on the landscape. If another agent is already there, they move again and again until they land on a free patch. There is no guarantee that this is a "happy" place for the agent just moved and this could affect the convergence of the model. However, with small landscapes this blind wandering through the patches has no effect on performance and coverges very quickly.

EXPERIMENTS
-----------
A number of experiments are described in Schelling's original article. An interesting modification of the model is the introduction of a third population.

NETLOGO FEATURES
----------------
This model is based on Segregation model in Netolog library model but has been rewritten from scratch to avoid hard coding parameters.

CREDITS AND REFERENCES
----------------------
Model developed by Fabrizio Iozzi, 2007, Universit Bocconi, Milano, Italy.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

link
true
0
Line -7500403 true 150 0 150 300

link direction
true
0
Line -7500403 true 150 150 30 225
Line -7500403 true 150 150 270 225

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>mean [count (turtles-on neighbors) with [color = yellow]] of turtles with [color = yellow]</metric>
    <metric>mean [count (turtles-on neighbors) with [color = cyan]] of turtles with [color = cyan]</metric>
    <steppedValueSet variable="perc-similar-wanted" first="30" step="1" last="70"/>
  </experiment>
  <experiment name="experiment2" repetitions="1" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>segregation-1</metric>
    <metric>segregation-2</metric>
    <steppedValueSet variable="perc-similar-wanted" first="30" step="1" last="50"/>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
